#ifndef DETECTORCONSTRUCTION_HH
#define DETECTORCONSTRUCTION_HH

#include "G4VUserDetectorConstruction.hh"
#include "globals.hh"
#include "G4Material.hh"
#include "G4ThreeVector.hh"
#include "G4VSensitiveDetector.hh"

class G4LogicalVolume;
class DetectorConstructionMessenger;

class DetectorConstruction : public G4VUserDetectorConstruction
{
public:
  DetectorConstruction();
  virtual ~DetectorConstruction();

  G4VPhysicalVolume* Construct();

  void UpdateGeometry();

  void SetNeutNumber(G4int val);
  void SetNeutSizeX(G4double val);
  void SetNeutSizeY(G4double val);
  void SetNeutSizeZ(G4double val);
  void SetNeutGap(G4double val);
  void SetNeutRotationMode(G4int val);

  void SetVetoNumber(G4int val);
  void SetVetoSizeX(G4double val);
  void SetVetoSizeY(G4double val);
  void SetVetoSizeZ(G4double val);
  void SetVetoGapY(G4double val);
  void SetVetoGapZ(G4double val);
  void SetVetoRotationMode(G4int val);

  void SetNeutLayerNumber(G4int val);
  void SetNeutLayerGap(G4double val);

  void SetNeutVetoGap(G4double val);

  void SetStuckNumber(G4int val);
  void SetStuckGap(G4double val);
  void SetStuckInterval(G4double val);

  void SetPositionX(G4double val);
  void SetPositionY(G4double val);
  void SetPositionZ(G4double val);

  //  void SetNeutMaterial(const G4String&);
  //  void SetWorldMaterial(const G4String&);

private:
  DetectorConstructionMessenger* fDetectorConstructionMessenger;

  G4int fNeutNumber;
  G4ThreeVector fNeutSize;
  G4double fNeutGap;
  G4int fNeutRotationMode;

  G4int fVetoNumber;
  G4ThreeVector fVetoSize;
  G4ThreeVector fVetoGap;
  G4int fVetoRotationMode;

  G4int fNeutLayerNumber;
  G4double fNeutLayerGap;

  // veto layer number is always 1

  G4double fNeutVetoGap;

  G4int fStuckNumber;
  G4double fStuckGap;

  G4ThreeVector fPosition;

  G4LogicalVolume* fLogicNeut;
  G4LogicalVolume* fLogicNeutLayer;
  G4LogicalVolume* fLogicStuck;
  G4LogicalVolume* fLogicVeto;
  G4LogicalVolume* fLogicVetoLayer;
  G4LogicalVolume* fLogicWorld;

  G4VSensitiveDetector* fScintillatorSD;

  G4Material* fNeutMaterial;
  G4Material* fNeutLayerMaterial;
  G4Material* fStuckMaterial;
  G4Material* fVetoMaterial;
  G4Material* fVetoLayerMaterial;
  G4Material* fWorldMaterial;
};

#endif

