
#include "G4RunManager.hh"
#include "G4UImanager.hh"
#include "G4PhysListFactory.hh"
#include "G4VModularPhysicsList.hh"

#ifdef G4VIS_USE
#include "G4VisExecutive.hh"
#include "G4TrajectoryDrawByParticleID.hh"
#endif

#ifdef G4UI_USE
#include "G4UIExecutive.hh"
#endif

#include "DetectorConstruction.hh"
#include "PrimaryGeneratorAction.hh"
#include "RunAction.hh"
#include "EventAction.hh"

#include "QBBC.hh"
#include "QGSP_BIC_HP.hh"
#include "QGSP_BIC_HP_NADEKO.hh"

// argv[1]: macro name
int main(int argc,char** argv)
{
  // Construct the default run manager
  G4RunManager* runManager = new G4RunManager();

  // set mandatory initialization classes
  runManager->SetUserInitialization(new DetectorConstruction());

  //  G4VModularPhysicsList* modularPhysicsList = new QBBC;
  //  G4VModularPhysicsList* modularPhysicsList = new QGSP_BIC_HP;
  G4VModularPhysicsList* modularPhysicsList = new QGSP_BIC_HP_NADEKO; // semi-manualy created physics list
  runManager->SetUserInitialization(modularPhysicsList);

  runManager->Initialize();

  // set user action classes
  runManager->SetUserAction(new PrimaryGeneratorAction());
  runManager->SetUserAction(new RunAction());
  runManager->SetUserAction(new EventAction());

#ifdef G4VIS_USE
  G4VisManager* visManager = 0;
  visManager = new G4VisExecutive;
  visManager->Initialize();

  G4TrajectoryDrawByParticleID* trajectoryModel = new G4TrajectoryDrawByParticleID;
  trajectoryModel->Set("proton","red");
  trajectoryModel->Set("antiproton","yellow");
  trajectoryModel->Set("neutron","green");
  trajectoryModel->Set("e-","blue");
  trajectoryModel->Set("e+","cyan");
  trajectoryModel->Set("gamma","white");
  trajectoryModel->Set("mu-","magenta");      
  trajectoryModel->Set("mu+","magenta");      
  trajectoryModel->Set("pi-","black");      
  trajectoryModel->Set("pi+","black");      

  visManager->RegisterModel(trajectoryModel);
#endif

  if(argc==1){  // interactive mode
#ifdef G4UI_USE
    G4UIExecutive* uiExecutive = new G4UIExecutive(argc, argv);
    uiExecutive->SessionStart();
    delete uiExecutive;
#endif
  }else{       // Batch mode
    G4String command = "/control/execute ";
    G4String fileName = argv[1];
    G4UImanager* UImanager = G4UImanager::GetUIpointer();
    UImanager->ApplyCommand(command+fileName);
  }

#ifdef G4VIS_USE
  delete visManager;
#endif

  delete runManager;

  return 0;
}
