
#include "NeutronDetectorMinEkineCuts.hh"

#include "G4Step.hh"
#include "G4UserLimits.hh"
#include "G4VParticleChange.hh"
#include "G4EnergyLossTables.hh"

NeutronDetectorMinEkineCuts::NeutronDetectorMinEkineCuts(const G4String& aName)
  : SpecialCuts(aName)
{
  if (verboseLevel>1) {
    G4cout << GetProcessName() << " is created "<< G4endl;
  }
  SetProcessType(fUserDefined);
}

NeutronDetectorMinEkineCuts::~NeutronDetectorMinEkineCuts()
{}

G4double NeutronDetectorMinEkineCuts::PostStepGetPhysicalInteractionLength(const G4Track& aTrack,
									   G4double ,
									   G4ForceCondition* condition)
{
  // condition is set to "Not Forced"
  *condition = NotForced;

  const G4DynamicParticle* aParticle = aTrack.GetDynamicParticle();
  G4ParticleDefinition* aParticleDef = aTrack.GetDefinition();

  G4double     proposedStep = DBL_MAX;
  if (aParticleDef->GetPDGCharge() != 0.0) {  // charged particles
    //min kinetic energy
    G4double eKine = aParticle->GetKineticEnergy();
    const G4MaterialCutsCouple* couple = aTrack.GetMaterialCutsCouple();

    G4double eMin = 1*MeV;
    G4double rangeNow = G4EnergyLossTables::GetRange(aParticleDef,eKine,couple);

    if (eKine < eMin ) {
      proposedStep = 0.;
    } else {
      G4double rangeMin = G4EnergyLossTables::GetRange(aParticleDef,eMin,couple);
      G4double temp = rangeNow - rangeMin;
      if (proposedStep > temp) proposedStep = temp;
    }
  }else{
    G4double eKine = aParticle->GetKineticEnergy();
    G4double eMin = 1*MeV;
    if (eKine < eMin ) {
      proposedStep = 0.;
    }
  } 

  return proposedStep;
}
