
#include "NeutronDetectorPhysics.hh"

#include "G4ParticleDefinition.hh"
#include "G4ProcessManager.hh"

#include "NeutronDetectorMaxTimeCuts.hh"
#include "NeutronDetectorMinEkineCuts.hh"
#include "G4StepLimiter.hh"

NeutronDetectorPhysics::NeutronDetectorPhysics(G4int ver)
  : G4VPhysicsConstructor("NeutronDetectorPhysics"), verbose(ver), 
    wasActivated(false)
{
  if(verbose > 1) { 
    G4cout << "### NeutronDetectorPhysics: " << GetPhysicsName() 
	   << G4endl; 
  }
}

NeutronDetectorPhysics::NeutronDetectorPhysics(const G4String&,
    G4int ver, G4bool, const G4String&)
  : G4VPhysicsConstructor("NeutronDetectorPhysics"), verbose(ver), 
    wasActivated(false)
{
  if(verbose > 1) { 
    G4cout << "### NeutronDetectorPhysics: " << GetPhysicsName() 
	   << G4endl; 
  }
}

NeutronDetectorPhysics::~NeutronDetectorPhysics()
{}

void NeutronDetectorPhysics::ConstructParticle()
{
  // no need to construct particle by this physics, I think...

//  G4cout << "NeutronDetectorPhysics::ConstructParticle" << G4endl;
//  G4MesonConstructor pMesonConstructor;
//  pMesonConstructor.ConstructParticle();
//
//  G4BaryonConstructor pBaryonConstructor;
//  pBaryonConstructor.ConstructParticle();
//
//  G4IonConstructor pConstructor;
//  pConstructor.ConstructParticle();  
}

void NeutronDetectorPhysics::ConstructProcess()
{
  if(wasActivated) { return; }
  wasActivated = true;

  //  G4cout << "NeutronDetectorPhysics::ConstructProcess" << G4endl;

  theParticleIterator->reset();
  while( (*theParticleIterator)() ){
    G4ParticleDefinition* particle = theParticleIterator->value();
    G4ProcessManager* pmanager = particle->GetProcessManager();
    G4String particleName = particle->GetParticleName();
    // time cuts for ALL particle
    //    if(particleName == "neutron") pmanager->AddDiscreteProcess(new NeutronDetectorMaxTimeCuts());
    pmanager->AddDiscreteProcess(new NeutronDetectorMaxTimeCuts());
    // Energy cuts to kill particles don't lose much energy 
    pmanager->AddDiscreteProcess(new NeutronDetectorMinEkineCuts());
 
    // Step limit applied to all particles: 
    pmanager->AddProcess(new G4StepLimiter, -1,-1,1);
  }
}


