
#include "PrimaryGeneratorAction.hh"
#include "PrimaryGeneratorActionMessenger.hh"

#include "G4Event.hh"
#include "G4ParticleTable.hh"
#include "G4ParticleDefinition.hh"
#include "G4ParticleGun.hh"

#include "G4PrimaryVertex.hh"
#include "G4PrimaryParticle.hh"

#include "TSimulationDataBeam.hh"

#include "TRandom3.h"
#include "TVector3.h"

PrimaryGeneratorAction::PrimaryGeneratorAction() : fStandardKineticEnergy(-1*MeV)
{
  fParticleGun = new G4ParticleGun(1);

  fBeamMessenger = new PrimaryGeneratorActionMessenger(this);

  Int_t seed = 126;
  fRandom = new TRandom3(seed);

  G4ThreeVector momentumDirection = G4ThreeVector(0, 0, 1);
  G4double kineticEnergy = 250*MeV;
  G4ThreeVector position = G4ThreeVector(0, 0, 0);

  fParticleGun->SetParticleMomentumDirection(momentumDirection);
  fParticleGun->SetParticleEnergy(kineticEnergy);
  fParticleGun->SetParticlePosition(position);
}

PrimaryGeneratorAction::~PrimaryGeneratorAction()
{
  delete fRandom;

  delete fParticleGun;
  delete fBeamMessenger;
}

void PrimaryGeneratorAction::GeneratePrimaries(G4Event* anEvent)
{
#if 0 // mono-energy mono-momentum beam
  /* nothing to do */
#elif 1 // mono-energy random-momentum beam
  G4ThreeVector momentumDirection = G4ThreeVector((fRandom->Uniform()-0.5)*0.05,
						  (fRandom->Uniform()-0.5)*0.05,
						  1);
  fParticleGun->SetParticleMomentumDirection(momentumDirection);
#endif

  fParticleGun->GeneratePrimaryVertex(anEvent);

  gSimulationDataBeamArray->clear();
  for(int i=0; i<anEvent->GetNumberOfPrimaryVertex(); ++i){
    G4PrimaryVertex* primaryVertex = anEvent->GetPrimaryVertex(i);
    for(int j=0; j<primaryVertex->GetNumberOfParticle(); ++j){
      G4int primaryParticleID = j+i*j;

      G4PrimaryParticle* primaryParticle = primaryVertex->GetPrimary(j);
      Double_t mass = primaryParticle->GetMass();
      Double_t charge = primaryParticle->GetCharge();
      Int_t pdgCode= primaryParticle->GetPDGcode();
      TVector3 momentum(primaryParticle->GetMomentum().x(),
			primaryParticle->GetMomentum().y(),
			primaryParticle->GetMomentum().z());
      TVector3 position(primaryVertex->GetX0(),
			primaryVertex->GetY0(),
			primaryVertex->GetZ0());
      Double_t time = primaryVertex->GetT0();

      TSimulationDataBeam simulationDataBeam(primaryParticleID, pdgCode, mass/MeV, charge/eplus, 1/MeV*momentum, 1/cm*position, time/ns);
      gSimulationDataBeamArray->push_back(simulationDataBeam);

#ifdef DEBUG
      std::cout << simulationDataBeam << std::endl;
#endif
    }
  }
}

void PrimaryGeneratorAction::SetStandardKineticEnergy(G4double val)
{
  fStandardKineticEnergy = val;
}
