
#include "RunAction.hh"
#include "RunActionMessenger.hh"

#include "G4UImanager.hh"
#include "G4VVisManager.hh"
#include "G4Run.hh"
#include "globals.hh"

#include "TSimulationDataBeam.hh"
#include "TSimulationDataScintillator.hh"
#include "TSimulationParameterNeutronDetector.hh"

#include "TFile.h"
#include "TTree.h"

#include <fstream>
#include <iomanip>
#include <sstream>

RunAction::RunAction() : fRunName("simulationdata")
{
  fRunMessenger = new RunActionMessenger(this);
}

RunAction::~RunAction()
{;}

void RunAction::BeginOfRunAction(const G4Run* aRun)
{
  G4int id = aRun->GetRunID();
  G4cout << "### Run " << id << " start" << G4endl;

  // accumulate data by ROOT
  std::ostringstream oss;
  oss << "root/" << fRunName << std::setw(4) << std::setfill('0') << id << ".root";
  // over write warning
  std::ifstream ifile(oss.str().c_str());
  if(ifile){
    G4cout << oss.str() << " already exists. over write[y], exit[n]>" << G4endl;    
    std::string yn;
    G4cin >> yn;
    if(!(yn == "y" || yn == "yes")) std::exit(1);
  }
  TFile* file = TFile::Open(oss.str().c_str(), "recreate");
  if(!file || file->IsZombie()) G4Exception(0, 0, FatalException, 0);

  // clean and create data buffer
  delete gSimulationDataBeamArray;
  delete gSimulationDataScintillatorArray;
  gSimulationDataBeamArray = new TSimulationDataBeamArray;
  gSimulationDataScintillatorArray = new TSimulationDataScintillatorArray;

  // create event data branch
  TTree* tree = new TTree("simTree", "simulation tree");
  tree->Branch("beam", gSimulationDataBeamArray);
  tree->Branch("neutronDetector", gSimulationDataScintillatorArray);

  // add parameter to root file
  file->Add(gSimulationParameterNeutronDetector);

#ifdef G4VIS_USE
  if(G4VVisManager::GetConcreteInstance()) G4UImanager::GetUIpointer()->ApplyCommand("/vis/scene/notifyHandlers");
#endif
}

void RunAction::EndOfRunAction(const G4Run* /*aRun*/)
{
  gFile->Write();

  TTree* tree;
  gDirectory->GetObject("simTree", tree);
  tree->ResetBranchAddresses();

#ifdef G4VIS_USE
  if(G4VVisManager::GetConcreteInstance()) G4UImanager::GetUIpointer()->ApplyCommand("/vis/viewer/update");
#endif
}

void RunAction::SetRunName(G4String val)
{
  fRunName = val;
}
